
#ifndef COLLECTION_H
#define COLLECTION_H

#include <iostream> /* std::ostream */
#include <vector>   /* std::vector */
#include <cassert>  /* assert */

#include "BigInteger.hh" /* BigInteger */


/* template declarations */
template <typename T>
class MyCollection {
    public:
        MyCollection(unsigned int max_size);
        ~MyCollection();

        void insert_elem(const T& element);
        T get_elem(unsigned int i) const;
        
        unsigned int max_size() const;
        unsigned int current_size() const;
    private:
        T* m_data;
        const unsigned int m_max_size;
        unsigned int m_current_size;
};


/* Non member template function definition */
template <typename T>
std::ostream& operator<<(std::ostream& os, const MyCollection<T>& collection);
template <typename T>
std::ostream& operator<<(std::ostream& os, const std::vector<T>& collection);

/* Specialization for BigInteger collections (operator << is not defined for BigIntegers, *
 * we need to use the 'bigIntegerToString' function defined in "BigIntegerUtils.hh".      */
std::ostream& operator<<(std::ostream& os, const MyCollection<BigInteger>& collection);
std::ostream& operator<<(std::ostream& os, const std::vector<BigInteger>& collection);



/* All template implementations */
template <typename T>
MyCollection<T>::MyCollection(unsigned int max_size):
    m_data(nullptr), m_max_size(max_size), m_current_size(0) {
        m_data = new T[m_max_size];
}


template <typename T>
MyCollection<T>::~MyCollection() {
    delete[] m_data;
}


template <typename T>
void MyCollection<T>::insert_elem(const T& elem){
    assert(m_current_size<m_max_size);
    m_data[m_current_size] = elem;
    m_current_size++;
}


template <typename T>
T MyCollection<T>::get_elem(unsigned int i) const{
    assert(i<m_max_size);
    return m_data[i];
}
        

template <typename T>
unsigned int MyCollection<T>::max_size() const {
    return m_max_size;
}


template <typename T>
unsigned int MyCollection<T>::current_size() const {
    return m_current_size;
}

template <typename T>
std::ostream& operator<<(std::ostream& os, const MyCollection<T>& collection) {
    for (unsigned int i= 0; i<collection.current_size(); i++) {
        os << collection.get_elem(i) << ' ';
    }
    return os;
}

template <typename T>
std::ostream& operator<<(std::ostream& os, const std::vector<T>& collection) {
    for (std::size_t i=0; i<collection.size(); i++) {
        os << collection[i] << ' ';
    }
    return os;
}

#endif /* end of include guard: COLLECTION_H */
